Class: Bake::Blocks::Compile
- Defined in:
- lib/blocks/compile.rb
Instance Attribute Summary collapse
-
#include_list ⇒ Object
readonly
Returns the value of attribute include_list.
-
#objects ⇒ Object
readonly
Returns the value of attribute objects.
Attributes inherited from BlockBase
Class Method Summary collapse
- .read_depfile(dep_filename, projDir, singeLine) ⇒ Object
-
.write_depfile(deps, dep_filename_conv) ⇒ Object
todo: move to toolchain util file.
Instance Method Summary collapse
- #calcCmdlineFile(object) ⇒ Object
- #calcDefines ⇒ Object
- #calcDepFile(object, type) ⇒ Object
- #calcDepFileConv(dep_filename) ⇒ Object
- #calcFileTcs ⇒ Object
- #calcFlags ⇒ Object
- #calcIncludes ⇒ Object
- #calcObjects ⇒ Object
- #calcSources(cleaning = false) ⇒ Object
- #clean ⇒ Object
- #compileFile(source) ⇒ Object
- #execute ⇒ Object
- #get_object_file(source) ⇒ Object
- #get_source_type(source) ⇒ Object
- #getBlocks(method) ⇒ Object
- #getDefines(compiler) ⇒ Object
- #getFlags(compiler) ⇒ Object
- #getSubBlocks(b, method) ⇒ Object
- #ignore?(type) ⇒ Boolean
-
#initialize(block, config, referencedConfigs, tcs) ⇒ Compile
constructor
A new instance of Compile.
- #mapInclude(inc, orgBlock) ⇒ Object
- #mutex ⇒ Object
- #needed?(source, object, type, dep_filename_conv) ⇒ Boolean
- #tcs4source(source) ⇒ Object
Methods inherited from BlockBase
#calcOutputDir, #check_config_file, #config_changed?, #defaultToolchainTime, isCmdLineEqual?, prepareOutput, #printCmd, #process_console_output, #process_result, writeCmdLineFile
Constructor Details
#initialize(block, config, referencedConfigs, tcs) ⇒ Compile
Returns a new instance of Compile.
16 17 18 19 20 21 22 23 24 25 |
# File 'lib/blocks/compile.rb', line 16 def initialize(block, config, referencedConfigs, tcs) super(block, config, referencedConfigs, tcs) @objects = [] @object_files = {} calcFileTcs calcIncludes calcDefines # not for files with changed tcs calcFlags # not for files with changed tcs end |
Instance Attribute Details
#include_list ⇒ Object (readonly)
Returns the value of attribute include_list.
14 15 16 |
# File 'lib/blocks/compile.rb', line 14 def include_list @include_list end |
#objects ⇒ Object (readonly)
Returns the value of attribute objects.
14 15 16 |
# File 'lib/blocks/compile.rb', line 14 def objects @objects end |
Class Method Details
.read_depfile(dep_filename, projDir, singeLine) ⇒ Object
200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 |
# File 'lib/blocks/compile.rb', line 200 def self.read_depfile(dep_filename, projDir, singeLine) deps = [] begin if singeLine File.readlines(dep_filename).each do |line| splitted = line.split(": ") deps << splitted[1].gsub(/[\\]/,'/') if splitted.length > 1 end else deps_string = File.read(dep_filename) deps_string = deps_string.gsub(/\\\n/,'') dep_splitted = deps_string.split(/([^\\]) /).each_slice(2).map(&:join)[2..-1] deps = dep_splitted.map { |d| d.gsub(/[\\] /,' ').gsub(/[\\]/,'/').strip }.delete_if {|d| d == "" } end rescue Exception Bake.formatter.printWarning("Could not read '#{dep_filename}'", projDir) return nil end deps end |
.write_depfile(deps, dep_filename_conv) ⇒ Object
todo: move to toolchain util file
222 223 224 225 226 227 228 229 230 231 232 233 234 235 |
# File 'lib/blocks/compile.rb', line 222 def self.write_depfile(deps, dep_filename_conv) if deps begin File.open(dep_filename_conv, 'wb') do |f| deps.each do |dep| f.puts(dep) end end rescue Exception Bake.formatter.printWarning("Could not write '#{dep_filename_conv}'", projDir) return nil end end end |
Instance Method Details
#calcCmdlineFile(object) ⇒ Object
80 81 82 |
# File 'lib/blocks/compile.rb', line 80 def calcCmdlineFile(object) object[0..-3] + ".cmdline" end |
#calcDefines ⇒ Object
485 486 487 488 489 490 |
# File 'lib/blocks/compile.rb', line 485 def calcDefines @define_array = {} [:CPP, :C, :ASM].each do |type| @define_array[type] = getDefines(@tcs[:COMPILER][type]) end end |
#calcDepFile(object, type) ⇒ Object
84 85 86 87 88 89 90 |
# File 'lib/blocks/compile.rb', line 84 def calcDepFile(object, type) dep_filename = nil if type != :ASM dep_filename = object[0..-3] + ".d" end dep_filename end |
#calcDepFileConv(dep_filename) ⇒ Object
92 93 94 |
# File 'lib/blocks/compile.rb', line 92 def calcDepFileConv(dep_filename) dep_filename + ".bake" end |
#calcFileTcs ⇒ Object
498 499 500 501 502 503 504 505 506 507 508 509 510 511 512 513 514 515 |
# File 'lib/blocks/compile.rb', line 498 def calcFileTcs @fileTcs = {} @config.files.each do |f| if (f.define.length > 0 or f.flags.length > 0) if f.name.include?"*" Bake.formatter.printWarning("Toolchain settings not allowed for file pattern #{f.name}", f) err_res = ErrorDesc.new err_res.file_name = @config.file_name err_res.line_number = f.line_number err_res.severity = ErrorParser::SEVERITY_WARNING err_res. = "Toolchain settings not allowed for file patterns" Bake::IDEInterface.instance.set_errors([err_res]) else @fileTcs[f.name] = integrateCompilerFile(Utils.deep_copy(@tcs),f) end end end end |
#calcFlags ⇒ Object
491 492 493 494 495 496 |
# File 'lib/blocks/compile.rb', line 491 def calcFlags @flag_array = {} [:CPP, :C, :ASM].each do |type| @flag_array[type] = getFlags(@tcs[:COMPILER][type]) end end |
#calcIncludes ⇒ Object
437 438 439 440 441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 |
# File 'lib/blocks/compile.rb', line 437 def calcIncludes @include_list = @config.includeDir.uniq.map do |dir| mapInclude(dir, @block) end getBlocks(:childs).each do |b| b.config.includeDir.each do |inc| if inc.inherit == true @include_list << mapInclude(inc, b) end end if b.config.respond_to?("includeDir") end getBlocks(:parents).each do |b| if b.config.respond_to?("includeDir") include_list_front = [] b.config.includeDir.each do |inc| if inc.inject == "front" include_list_front << mapInclude(inc, b) elsif inc.inject == "back" @include_list << mapInclude(inc, b) elsif inc.infix == "front" include_list_front << mapInclude(inc, b) elsif inc.infix == "back" @include_list << mapInclude(inc, b) end end @include_list = include_list_front + @include_list end end @include_list = @include_list.flatten.uniq @include_array = {} [:CPP, :C, :ASM].each do |type| @include_array[type] = @include_list.map {|k| "#{@tcs[:COMPILER][type][:INCLUDE_PATH_FLAG]}#{k}"} end end |
#calcObjects ⇒ Object
334 335 336 337 338 339 340 341 342 343 |
# File 'lib/blocks/compile.rb', line 334 def calcObjects @source_files.each do |source| type = get_source_type(source) if not type.nil? object = get_object_file(source) @object_files[source] = object @objects << object end end end |
#calcSources(cleaning = false) ⇒ Object
345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 |
# File 'lib/blocks/compile.rb', line 345 def calcSources(cleaning = false) Dir.chdir(@projectDir) do @source_files = [] exclude_files = Set.new @config.excludeFiles.each do |p| Dir.glob(p.name).each {|f| exclude_files << f} end source_files = Set.new @config.files.each do |sources| p = sources.name res = Dir.glob(p).sort if res.length == 0 and cleaning == false if not p.include?"*" and not p.include?"?" Bake.formatter.printError("Source file '#{p}' not found", sources) raise SystemCommandFailed.new elsif Bake..verbose >= 1 Bake.formatter.printInfo("Source file pattern '#{p}' does not match to any file", sources) end end res.each do |f| next if exclude_files.include?(f) source_files << f end end if Bake..filename source_files.keep_if do |source| source.include?Bake..filename end if source_files.length == 0 and cleaning == false Bake.formatter.printInfo("#{Bake..filename} does not match to any source", @config) end end @source_files = source_files.sort.to_a if Bake..eclipseOrder # directories reverse order, files in directories in alphabetical order dirs = [] filemap = {} @source_files.reverse.each do |o| d = File.dirname(o) if filemap.include?(d) filemap[d] << o else filemap[d] = [o] dirs << d end end @source_files = [] dirs.each do |d| filemap[d].reverse.each do |f| @source_files << f end end end end return (not @source_files.empty?) end |
#clean ⇒ Object
303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 |
# File 'lib/blocks/compile.rb', line 303 def clean if Bake..filename or Bake..analyze Dir.chdir(@projectDir) do calcSources(true) @source_files.each do |source| type = get_source_type(source) next if type.nil? object = get_object_file(source) if File.exist?object puts "Deleting file #{object}" if Bake..verbose >= 2 FileUtils.rm_rf(object) end if not Bake..analyze dep_filename = calcDepFile(object, type) if dep_filename and File.exist?dep_filename puts "Deleting file #{dep_filename}" if Bake..verbose >= 2 FileUtils.rm_rf(dep_filename) end cmdLineFile = calcCmdlineFile(object) if File.exist?cmdLineFile puts "Deleting file #{cmdLineFile}" if Bake..verbose >= 2 FileUtils.rm_rf(cmdLineFile) end end end end end return true end |
#compileFile(source) ⇒ Object
104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 |
# File 'lib/blocks/compile.rb', line 104 def compileFile(source) type = get_source_type(source) return true if type.nil? object = @object_files[source] dep_filename = calcDepFile(object, type) dep_filename_conv = calcDepFileConv(dep_filename) if type != :ASM cmdLineCheck = false cmdLineFile = calcCmdlineFile(object) return true if ignore?(type) reason = needed?(source, object, type, dep_filename_conv) if not reason cmdLineCheck = true # currently this returns always != nil reason = config_changed?(cmdLineFile) end return true unless reason if @fileTcs.include?(source) compiler = @fileTcs[source][:COMPILER][type] defines = getDefines(compiler) flags = getFlags(compiler) else compiler = @tcs[:COMPILER][type] defines = @define_array[type] flags = @flag_array[type] end includes = @include_array[type] if Bake..prepro and compiler[:PREPRO_FLAGS] == "" Bake.formatter.printError("Error: No preprocessor option available for " + source) raise SystemCommandFailed.new end cmd = Utils.flagSplit(compiler[:COMMAND], false) cmd += compiler[:COMPILE_FLAGS].split(" ") if dep_filename cmd += @tcs[:COMPILER][type][:DEP_FLAGS].split(" ") if @tcs[:COMPILER][type][:DEP_FLAGS_FILENAME] if @tcs[:COMPILER][type][:DEP_FLAGS_SPACE] cmd << dep_filename else if dep_filename.include?" " cmd[cmd.length-1] << "\"" + dep_filename + "\"" else cmd[cmd.length-1] << dep_filename end end end end cmd += compiler[:PREPRO_FLAGS].split(" ") if Bake..prepro cmd += flags cmd += includes cmd += defines offlag = compiler[:OBJECT_FILE_FLAG] offlag = compiler[:PREPRO_FILE_FLAG] if compiler[:PREPRO_FILE_FLAG] and Bake..prepro if compiler[:OBJ_FLAG_SPACE] cmd << offlag cmd << object else if object.include?" " cmd << offlag + "\"" + object + "\"" else cmd << offlag + object end end cmd << source if Bake..cc2j_filename Blocks::CC2J << { :directory => @projectDir, :command => cmd, :file => source } end return true if cmdLineCheck and BlockBase.isCmdLineEqual?(cmd, cmdLineFile) BlockBase.prepareOutput(object) BlockBase.writeCmdLineFile(cmd, cmdLineFile) success, consoleOutput = ProcessHelper.run(cmd, false, false) outputType = Bake..analyze ? "Analyzing" : (Bake..prepro ? "Preprocessing" : "Compiling") incList = process_result(cmd, consoleOutput, compiler[:ERROR_PARSER], "#{outputType} #{source}", reason, success) if type != :ASM and not Bake..analyze and not Bake..prepro incList = Compile.read_depfile(dep_filename, @projectDir, @tcs[:COMPILER][:DEP_FILE_SINGLE_LINE]) if incList.nil? Compile.write_depfile(incList, dep_filename_conv) end check_config_file end |
#execute ⇒ Object
241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 |
# File 'lib/blocks/compile.rb', line 241 def execute Dir.chdir(@projectDir) do calcSources calcObjects @error_strings = {} compileJobs = Multithread::Jobs.new(@source_files) do |jobs| while source = jobs.get_next_or_nil do if (jobs.failed and Bake..stopOnFirstError) or Bake::IDEInterface.instance.get_abort break end s = StringIO.new tmp = Thread.current[:stdout] Thread.current[:stdout] = s unless tmp result = false begin compileFile(source) result = true rescue Bake::SystemCommandFailed => scf # normal compilation error rescue SystemExit => exSys rescue Exception => ex1 if not Bake::IDEInterface.instance.get_abort Bake.formatter.printError("Error: #{ex1.}") puts ex1.backtrace if Bake..debug end end jobs.set_failed if not result Thread.current[:stdout] = tmp mutex.synchronize do if s.string.length > 0 if Bake..stopOnFirstError and not result @error_strings[source] = s.string else puts s.string end end end end end compileJobs.join # can only happen in case of bail_on_first_error. # if not sorted, it may be confusing when builing more than once and the order of the error appearances changes from build to build # (it is not deterministic which file compilation finishes first) @error_strings.sort.each {|es| puts es[1]} raise SystemCommandFailed.new if compileJobs.failed end return true end |
#get_object_file(source) ⇒ Object
27 28 29 30 31 32 33 |
# File 'lib/blocks/compile.rb', line 27 def get_object_file(source) # until now all OBJECT_FILE_ENDING are equal in all three types adaptedSource = source.chomp(File.extname(source)).gsub(/\.\./, "##") + (Bake..prepro ? ".i" : @tcs[:COMPILER][:CPP][:OBJECT_FILE_ENDING]) return adaptedSource if File.is_absolute?source File.join([@output_dir, adaptedSource]) end |
#get_source_type(source) ⇒ Object
96 97 98 99 100 101 102 |
# File 'lib/blocks/compile.rb', line 96 def get_source_type(source) ex = File.extname(source) [:CPP, :C, :ASM].each do |t| return t if @tcs[:COMPILER][t][:SOURCE_FILE_ENDINGS].include?(ex) end nil end |
#getBlocks(method) ⇒ Object
415 416 417 418 419 |
# File 'lib/blocks/compile.rb', line 415 def getBlocks(method) @otherBlocks = [] getSubBlocks(@block, method) return @otherBlocks end |
#getDefines(compiler) ⇒ Object
477 478 479 |
# File 'lib/blocks/compile.rb', line 477 def getDefines(compiler) compiler[:DEFINES].map {|k| "#{compiler[:DEFINE_FLAG]}#{k}"} end |
#getFlags(compiler) ⇒ Object
481 482 483 |
# File 'lib/blocks/compile.rb', line 481 def getFlags(compiler) Bake::Utils::flagSplit(compiler[:FLAGS],true) end |
#getSubBlocks(b, method) ⇒ Object
406 407 408 409 410 411 412 413 |
# File 'lib/blocks/compile.rb', line 406 def getSubBlocks(b, method) b.send(method).each do |child_b| if not @otherBlocks.include?child_b @otherBlocks << child_b getSubBlocks(child_b, method) end end end |
#ignore?(type) ⇒ Boolean
35 36 37 |
# File 'lib/blocks/compile.rb', line 35 def ignore?(type) Bake..linkOnly or (Bake..prepro and type == ASM) end |
#mapInclude(inc, orgBlock) ⇒ Object
421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 |
# File 'lib/blocks/compile.rb', line 421 def mapInclude(inc, orgBlock) if inc.name == "___ROOTS___" return Bake..roots.map { |r| File.rel_from_to_project(@projectDir,r,false) } end i = orgBlock.convPath(inc) if orgBlock != @block if not File.is_absolute?(i) i = File.rel_from_to_project(@projectDir,orgBlock.config.parent.get_project_dir) + i end end Pathname.new(i).cleanpath end |
#mutex ⇒ Object
237 238 239 |
# File 'lib/blocks/compile.rb', line 237 def mutex @mutex ||= Mutex.new end |
#needed?(source, object, type, dep_filename_conv) ⇒ Boolean
39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 |
# File 'lib/blocks/compile.rb', line 39 def needed?(source, object, type, dep_filename_conv) return "because analyzer toolchain is configured" if Bake..analyze return "because prepro was specified and source is no assembler file" if Bake..prepro return "because object does not exist" if not File.exist?(object) oTime = File.mtime(object) return "because source is newer than object" if oTime < File.mtime(source) if type != :ASM return "because dependency file does not exist" if not File.exist?(dep_filename_conv) begin File.readlines(dep_filename_conv).map{|line| line.strip}.each do |dep| if not File.exist?(dep) # we need a hack here. with some windows configurations the compiler prints unix paths # into the dep file which cannot be found easily. this will be true for system includes, # e.g. /usr/lib/...xy.h if (Bake::Utils::OS.windows? and dep.start_with?"/") or (not Bake::Utils::OS.windows? and dep.length > 1 and dep[1] == ":") puts "Dependency header file #{dep} ignored!" if Bake..debug else return "because dependent header #{dep} does not exist" end else return "because dependent header #{dep} is newer than object" if oTime < File.mtime(dep) end end rescue Exception => ex if Bake..debug puts "While reading #{dep_filename_conv}:" puts ex. puts ex.backtrace end return "because dependency file could not be loaded" end end false end |
#tcs4source(source) ⇒ Object
517 518 519 |
# File 'lib/blocks/compile.rb', line 517 def tcs4source(source) @fileTcs[source] || @tcs end |